@@ -2,7 +2,7 @@
 * Software License Agreement (BSD License)                                                               *
 * Author: Sebastien Decugis <sdecugis@freediameter.net>							 *
 *													 *
-* Copyright (c) 2020, WIDE Project and NICT								 *
+* Copyright (c) 2023, WIDE Project and NICT								 *
 * All rights reserved.											 *
 * 													 *
 * Redistribution and use of this software in source and binary forms, with or without modification, are  *
@@ -808,8 +808,8 @@ int rgw_clients_create_origin(struct rgw_radius_msg_meta *msg, struct rgw_client
 				/* It is a valid alias, save it */
 				CHECK_MALLOC( cli->aliases = realloc(cli->aliases, (cli->aliases_nb + 1) * sizeof(cli->aliases[0])) );
 				
-				CHECK_MALLOC( cli->aliases[cli->aliases_nb + 1].name = os0dup(nas_id + 1, nas_id_len ) );
-				cli->aliases[cli->aliases_nb + 1].len = nas_id_len;
+				CHECK_MALLOC( cli->aliases[cli->aliases_nb].name = os0dup(nas_id + 1, nas_id_len ) );
+				cli->aliases[cli->aliases_nb].len = nas_id_len;
 
 				cli->aliases_nb ++;
 				TRACE_DEBUG(FULL, "Saved valid alias for client: '%.*s' -> '%s'", (int)nas_id_len, (char *)(nas_id + 1), cli->fqdn);
@@ -1129,7 +1129,7 @@ int rgw_client_finish_send(struct radius_msg ** msg, struct rgw_radius_msg_meta
 	return 0;
 }
 
-/* Call this function when a RADIUS request has explicitely no answer (mainly accounting) so 
+/* Call this function when a RADIUS request has explicitly no answer (mainly accounting) so 
 that we purge the duplicate cache and allow further message to be translated again.
 This is useful for example when a temporary error occurred in Diameter (like UNABLE_TO_DELIVER) */
 int rgw_client_finish_nosend(struct rgw_radius_msg_meta * req, struct rgw_client * cli)
