@@ -2,7 +2,7 @@
 * Software License Agreement (BSD License)                                                               *
 * Author: Sebastien Decugis <sdecugis@freediameter.net>							 *
 *													 *
-* Copyright (c) 2019, WIDE Project and NICT								 *
+* Copyright (c) 2023, WIDE Project and NICT								 *
 * All rights reserved.											 *
 * 													 *
 * Redistribution and use of this software in source and binary forms, with or without modification, are  *
@@ -141,12 +141,13 @@ awake:
 /* Allow the state machines to start */
 int fd_psm_start()
 {
+	int ret = 0;
 	TRACE_ENTRY("");
 	CHECK_POSIX( pthread_mutex_lock(&started_mtx) );
 	started = 1;
-	CHECK_POSIX( pthread_cond_broadcast(&started_cnd) );
+	ret = pthread_cond_broadcast(&started_cnd);
 	CHECK_POSIX( pthread_mutex_unlock(&started_mtx) );
-	return 0;
+	return ret;
 }
 
 
@@ -357,6 +358,8 @@ void fd_psm_cleanup(struct fd_peer * peer, int terminate)
 		CHECK_FCT_DO( fd_psm_change_state(peer, STATE_CLOSED), /* continue */ );
 	}
 
+	fd_p_sr_on_disconnect(&peer->p_sr);
+
 	fd_p_cnx_abort(peer, terminate);
 
 	fd_p_ce_clear_cnx(peer, NULL);
@@ -638,7 +641,10 @@ psm_loop:
 			case CC_DISCONNECT_PEER:
 				CHECK_FCT_DO( fd_p_dp_handle(&msg, (hdr->msg_flags & CMD_FLAG_REQUEST), peer), goto psm_reset );
 				if (fd_peer_getstate(peer) == STATE_CLOSING)
+				{
+					fd_hook_call(HOOK_PEER_CONNECT_FAILED, NULL, peer, "Connection closed", NULL);
 					goto psm_end;
+				}
 
 				break;
 
@@ -716,9 +722,11 @@ psm_loop:
 
 			case STATE_CLOSING:
 				/* We sent a DPR so we are terminating, do not wait for DPA */
+				fd_hook_call(HOOK_PEER_CONNECT_FAILED, NULL, peer, "Connection closed", NULL);
 				goto psm_end;
 
 			case STATE_CLOSING_GRACE:
+				fd_hook_call(HOOK_PEER_CONNECT_FAILED, NULL, peer, "Connection closed", NULL);
 				if (peer->p_flags.pf_localterm) /* initiated here */
 					goto psm_end;
 
@@ -851,13 +859,19 @@ psm_loop:
 			case STATE_WAITCNXACK:
 			case STATE_WAITCEA:
 				fd_hook_call(HOOK_PEER_CONNECT_FAILED, NULL, peer, "Timeout while waiting for remote peer", NULL);
+				/* Destroy the connection, restart the timer to a new connection attempt */
+				fd_psm_next_timeout(peer, 1, peer->p_hdr.info.config.pic_tctimer ?: fd_g_config->cnf_timer_tc);
+				goto psm_reset;
+
 			case STATE_CLOSING:
+				fd_hook_call(HOOK_PEER_CONNECT_FAILED, NULL, peer, "Connection closed", NULL);
 				/* Destroy the connection, restart the timer to a new connection attempt */
 				fd_psm_next_timeout(peer, 1, peer->p_hdr.info.config.pic_tctimer ?: fd_g_config->cnf_timer_tc);
 				goto psm_reset;
 
 			case STATE_CLOSING_GRACE:
 				/* The grace period is completed, now close */
+				fd_hook_call(HOOK_PEER_CONNECT_FAILED, NULL, peer, "Connection closed", NULL);
 				if (peer->p_flags.pf_localterm)
 					goto psm_end;
 
@@ -900,6 +914,10 @@ psm_end:
 	pthread_cleanup_pop(1); /* set STATE_ZOMBIE */
 	peer->p_psm = (pthread_t)NULL;
 	pthread_detach(pthread_self());
+	/* any problems this connection had are gone with it now */
+	peer->p_flags.pf_cnx_pb = 0;
+	/* we will not get any replies to DWRs on this connection any longer */
+	peer->p_flags.pf_dw_pending = 0;
 	return NULL;
 }
 
