@@ -2,7 +2,7 @@
 * Software License Agreement (BSD License)                                                               *
 * Author: Sebastien Decugis <sdecugis@freediameter.net>							 *
 *													 *
-* Copyright (c) 2019, WIDE Project and NICT								 *
+* Copyright (c) 2023, WIDE Project and NICT								 *
 * All rights reserved.											 *
 * 													 *
 * Redistribution and use of this software in source and binary forms, with or without modification, are  *
@@ -287,7 +287,7 @@ static int score_destination_avp(void * cbdata, struct msg ** pmsg, struct fd_li
 			c->score += FD_SCORE_FINALDEST;
 		} else {
 			if (dr && !fd_os_almostcasesrch(dr->os.data, dr->os.len, c->realm, c->realmlen, NULL) ) {
-				/* The candidate's realm matchs the Destination-Realm */
+				/* The candidate's realm matches the Destination-Realm */
 				c->score += FD_SCORE_REALM;
 			}
 		}
@@ -443,6 +443,8 @@ static int msg_dispatch(struct msg * msg)
 	char * ec = NULL;
 	char * em = NULL;
 	struct msg *msgptr = msg, *error = NULL;
+	DiamId_t qry_src = NULL;
+	size_t qry_src_len = 0;
 
 	/* Read the message header */
 	CHECK_FCT( fd_msg_hdr(msg, &hdr) );
@@ -485,6 +487,9 @@ static int msg_dispatch(struct msg * msg)
 		/* Retrieve the corresponding query */
 		CHECK_FCT( fd_msg_answ_getq( msgptr, &qry ) );
 
+		/* Check where that qry was coming from */
+		CHECK_FCT( fd_msg_source_get( qry, &qry_src, &qry_src_len ) );
+
 		/* Retrieve any registered handler */
 		CHECK_FCT( fd_msg_anscb_get( qry, &anscb, NULL, &data ) );
 
@@ -514,7 +519,7 @@ static int msg_dispatch(struct msg * msg)
 		switch ( action ) {
 			case DISP_ACT_CONT:
 				/* No callback has handled the message, let's reply with a generic error or relay it */
-				if (!fd_g_config->cnf_flags.no_fwd) {
+				if ((!fd_g_config->cnf_flags.no_fwd) && (is_req || qry_src)) {
 					/* requeue to fd_g_outgoing */
 					fd_hook_call(HOOK_MESSAGE_ROUTING_FORWARD, msgptr, NULL, NULL, fd_msg_pmdl_get(msgptr));
 					CHECK_FCT( fd_fifo_post(fd_g_outgoing, &msgptr) );
@@ -737,10 +742,17 @@ static int msg_rt_in(struct msg * msg)
 			return 0;
 		}
 
-		/* If the message is explicitely for someone else */
+		/* If the message is explicitly for someone else */
 		if ((is_dest_host == NO) || (is_dest_realm == NO)) {
 			if (fd_g_config->cnf_flags.no_fwd) {
-				fd_hook_call(HOOK_MESSAGE_ROUTING_ERROR, msgptr, NULL, "Message for another realm/host", fd_msg_pmdl_get(msgptr));
+				char * error;
+				if (is_dest_host == NO) {
+					error = "Message for another host";
+				}
+				else if (is_dest_realm == NO) {
+					error = "Message for another realm";
+				}
+				fd_hook_call(HOOK_MESSAGE_ROUTING_ERROR, msgptr, NULL, error, fd_msg_pmdl_get(msgptr));
 				CHECK_FCT( return_error( &msgptr, "DIAMETER_UNABLE_TO_DELIVER", "I am not a Diameter agent", NULL) );
 				return 0;
 			}
@@ -767,7 +779,7 @@ static int msg_rt_in(struct msg * msg)
 			}
 
 			if (is_local_app == YES) {
-				/* Handle localy since we are able to */
+				/* Handle locally since we are able to */
 				fd_hook_call(HOOK_MESSAGE_ROUTING_LOCAL, msgptr, NULL, NULL, fd_msg_pmdl_get(msgptr));
 				CHECK_FCT(fd_fifo_post(fd_g_local, &msgptr) );
 				return 0;
@@ -792,7 +804,7 @@ static int msg_rt_in(struct msg * msg)
 		CHECK_FCT( fd_msg_source_get( qry, &qry_src, NULL ) );
 
 		if ((!qry_src) && (!is_err)) {
-			/* The message is a normal answer to a request issued localy, we do not call the callbacks chain on it. */
+			/* The message is a normal answer to a request issued locally, we do not call the callbacks chain on it. */
 			fd_hook_call(HOOK_MESSAGE_ROUTING_LOCAL, msgptr, NULL, NULL, fd_msg_pmdl_get(msgptr));
 			CHECK_FCT(fd_fifo_post(fd_g_local, &msgptr) );
 			return 0;
@@ -962,7 +974,7 @@ static int msg_rt_out(struct msg * msg)
 		CHECK_FCT( fd_msg_rt_associate ( msgptr, rtd ) );
 	}
 
-	/* Note: we reset the scores and pass the message to the callbacks, maybe we could re-use the saved scores when we have received an error ? -- TODO */
+	/* Note: we reset the scores and pass the message to the callbacks, maybe we could reuse the saved scores when we have received an error ? -- TODO */
 
 	/* Ok, we have our list in rtd now, let's (re)initialize the scores */
 	fd_rtd_candidate_extract(rtd, &candidates, FD_SCORE_INI);
@@ -1127,7 +1139,7 @@ fatal_error:
 	CHECK_FCT_DO(fd_core_shutdown(), );
 	
 end:	
-	; /* noop so that we get rid of "label at end of compund statement" warning */
+	; /* noop so that we get rid of "label at end of compound statement" warning */
 	/* Mark the thread as terminated */
 	pthread_cleanup_pop(1);
 	return NULL;
