@@ -2,7 +2,7 @@
 * Software License Agreement (BSD License)                                                               *
 * Author: Sebastien Decugis <sdecugis@freediameter.net>							 *
 *													 *
-* Copyright (c) 2020, WIDE Project and NICT								 *
+* Copyright (c) 2023, WIDE Project and NICT								 *
 * All rights reserved.											 *
 * 													 *
 * Redistribution and use of this software in source and binary forms, with or without modification, are  *
@@ -39,7 +39,7 @@
 #include <netinet/sctp.h>
 #include <sys/uio.h>
 
-/* Size of buffer to receive ancilliary data. May need to be enlarged if more sockopt are set... */
+/* Size of buffer to receive ancillary data. May need to be enlarged if more sockopt are set... */
 #ifndef CMSG_BUF_LEN
 #define CMSG_BUF_LEN	1024
 #endif /* CMSG_BUF_LEN */
@@ -399,7 +399,7 @@ static int fd_setsockopt_prebind(int sk)
 		struct sctp_event_subscribe event;
 
 		memset(&event, 0, sizeof(event));
-		event.sctp_data_io_event	= 1;	/* to receive the stream ID in SCTP_SNDRCV ancilliary data on message reception */
+		event.sctp_data_io_event	= 1;	/* to receive the stream ID in SCTP_SNDRCV ancillary data on message reception */
 		event.sctp_association_event	= 0;	/* new or closed associations (mostly for one-to-many style sockets) */
 		event.sctp_address_event	= 1;	/* address changes */
 		event.sctp_send_failure_event	= 1;	/* delivery failures */
@@ -532,29 +532,29 @@ static int fd_setsockopt_prebind(int sk)
 	/* SCTP_EXPLICIT_EOR: we assume implicit EOR in freeDiameter, so let's ensure this is known by the stack */
 	#ifdef SCTP_EXPLICIT_EOR
 	{
-		int bool;
+		int _bool;
 		
 		if (TRACE_BOOL(ANNOYING)) {
-			sz = sizeof(bool);
+			sz = sizeof(_bool);
 			/* Read socket defaults */
-			CHECK_SYS(  getsockopt(sk, IPPROTO_SCTP, SCTP_EXPLICIT_EOR, &bool, &sz)  );
-			if (sz != sizeof(bool))
+			CHECK_SYS(  getsockopt(sk, IPPROTO_SCTP, SCTP_EXPLICIT_EOR, &_bool, &sz)  );
+			if (sz != sizeof(_bool))
 			{
-				TRACE_DEBUG(INFO, "Invalid size of socket option: %d / %d", sz, (socklen_t)sizeof(bool));
+				TRACE_DEBUG(INFO, "Invalid size of socket option: %d / %d", sz, (socklen_t)sizeof(_bool));
 				return ENOTSUP;
 			}
-			fd_log_debug( "Def SCTP_EXPLICIT_EOR value : %s", bool ? "true" : "false");
+			fd_log_debug( "Def SCTP_EXPLICIT_EOR value : %s", _bool ? "true" : "false");
 		}
 
-		bool = 0;
+		_bool = 0;
 		
 		/* Set the option to the socket */
-		CHECK_SYS(  setsockopt(sk, IPPROTO_SCTP, SCTP_EXPLICIT_EOR, &bool, sizeof(bool))  );
+		CHECK_SYS(  setsockopt(sk, IPPROTO_SCTP, SCTP_EXPLICIT_EOR, &_bool, sizeof(_bool))  );
 		
 		if (TRACE_BOOL(ANNOYING)) {
 			/* Check new values */
-			CHECK_SYS(  getsockopt(sk, IPPROTO_SCTP, SCTP_EXPLICIT_EOR, &bool, &sz)  );
-			fd_log_debug( "New SCTP_EXPLICIT_EOR value : %s", bool ? "true" : "false");
+			CHECK_SYS(  getsockopt(sk, IPPROTO_SCTP, SCTP_EXPLICIT_EOR, &_bool, &sz)  );
+			fd_log_debug( "New SCTP_EXPLICIT_EOR value : %s", _bool ? "true" : "false");
 		}
 	}
 	#else /* SCTP_EXPLICIT_EOR */
@@ -619,10 +619,10 @@ static int fd_setsockopt_prebind(int sk)
 	
 	#ifdef SCTP_RECVRCVINFO /* Replaces SCTP_SNDRCV */
 	{
-		int bool = 1;
+		int _bool = 1;
 		
 		/* Set the option to the socket */
-		CHECK_SYS(  setsockopt(sk, IPPROTO_SCTP, SCTP_RECVRCVINFO, &bool, sizeof(bool))  );
+		CHECK_SYS(  setsockopt(sk, IPPROTO_SCTP, SCTP_RECVRCVINFO, &_bool, sizeof(_bool))  );
 		
 	}
 	#else /* SCTP_RECVRCVINFO */
@@ -969,7 +969,7 @@ int fd_sctp_client( int *sock, int no_ip6, uint16_t port, struct fd_list * list,
 #else /* SCTP_CONNECTX_4_ARGS */
 	ret = sctp_connectx(*sock, sar.sa, count);
 #endif /* SCTP_CONNECTX_4_ARGS */
-	/* back to normal cancelation behabior */
+	/* back to normal cancellation behavior */
 	pthread_setcanceltype(PTHREAD_CANCEL_DEFERRED, NULL);
 	
 	if (ret < 0) {
@@ -1108,6 +1108,9 @@ stop:
 	return 0;
 }
 
+#define FD_SCTP_UNSECURED_DIAMETER_PPID 46
+#define FD_SCTP_SECURED_DIAMETER_PPID 47
+
 /* Send a vector over a specified stream */
 ssize_t fd_sctp_sendstrv(struct cnxctx * conn, uint16_t strid, const struct iovec *iov, int iovcnt)
 {
@@ -1138,10 +1141,18 @@ ssize_t fd_sctp_sendstrv(struct cnxctx * conn, uint16_t strid, const struct iove
 	hdr->cmsg_type  = SCTP_SNDRCV;
 	sndrcv = (struct sctp_sndrcvinfo *)CMSG_DATA(hdr);
 	sndrcv->sinfo_stream = strid;
+	if (!fd_cnx_teststate(conn, CC_STATUS_TLS))
+		sndrcv->sinfo_ppid = htonl(FD_SCTP_UNSECURED_DIAMETER_PPID);
+	else
+		sndrcv->sinfo_ppid = htonl(FD_SCTP_SECURED_DIAMETER_PPID);
 #else /* OLD_SCTP_SOCKET_API */
 	hdr->cmsg_type  = SCTP_SNDINFO;
 	sndinf = (struct sctp_sndinfo *)CMSG_DATA(hdr);
 	sndinf->snd_sid = strid;
+	if (!fd_cnx_teststate(conn, CC_STATUS_TLS))
+		sndinf->snd_ppid = htonl(FD_SCTP_UNSECURED_DIAMETER_PPID);
+	else
+		sndinf->snd_ppid = htonl(FD_SCTP_SECURED_DIAMETER_PPID);
 #endif /* OLD_SCTP_SOCKET_API */
 	/* note : we could store other data also, for example in .sinfo_ppid for remote peer or in .sinfo_context for errors. */
 	
